import { d as defaultFormatters, c as createFormat, a as createCustomFormatFn, e as extend, b as createParse, f as defaultParsers, p as parseOrThrowWrapper } from '../../default-parsers-50f9e074.js';
export { k as addDate, j as addHours, g as addMilliseconds, i as addMinutes, l as addMonth, h as addSeconds, m as addYear, T as asValidDateOrNull, E as diffCalendarDate, D as diffCalendarMonth, C as diffCalendarYear, z as diffDate, y as diffHours, v as diffMilliseconds, x as diffMinutes, A as diffMonth, I as diffPreciseDate, H as diffPreciseHours, G as diffPreciseMinutes, J as diffPreciseMonth, F as diffPreciseSeconds, K as diffPreciseYear, w as diffSeconds, B as diffYear, L as formatDateIso, M as formatDateTimeIso, N as formatLocalIso, R as fromDate, S as fromDateOrThrow, Q as isValidDate, U as newValidDate, V as newValidDateOrThrow, O as parseIso, P as parseIsoOrThrow, q as resetDate, s as resetHours, o as resetISOWeek, t as resetMinutes, n as resetMonth, u as resetSeconds, r as resetYear } from '../../default-parsers-50f9e074.js';

// Note: in English, the names of days of the week and months are capitalized.
// If you are making a new locale based on this one, check if the same is true for the language you're working on.
// Generally, formatted dates should look like they are in the middle of a sentence,
// e.g. in Spanish language the weekdays and months should be in the lowercase.
var months3char = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
var monthsFull = [
    'January',
    'February',
    'March',
    'April',
    'May',
    'June',
    'July',
    'August',
    'September',
    'October',
    'November',
    'December',
];
var weekdays2char = ['Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa'];
var weekdays3char = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
var weekdaysFull = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
var meridiemUppercase = ['AM', 'PM'];
var meridiemLowercase = ['am', 'pm'];
var meridiemFull = ['a.m.', 'p.m.'];
var formatters = {
    // Month: Jan, Feb, ..., Dec
    MMM: function (date) { return months3char[date.getMonth()]; },
    // Month: January, February, ..., December
    MMMM: function (date) { return monthsFull[date.getMonth()]; },
    // Day of week: Su, Mo, ..., Sa
    dd: function (date) { return weekdays2char[date.getDay()]; },
    // Day of week: Sun, Mon, ..., Sat
    ddd: function (date) { return weekdays3char[date.getDay()]; },
    // Day of week: Sunday, Monday, ..., Saturday
    dddd: function (date) { return weekdaysFull[date.getDay()]; },
    // AM, PM
    A: function (date) { return meridiemUppercase[date.getHours() < 12 ? 0 : 1]; },
    // am, pm
    a: function (date) { return meridiemLowercase[date.getHours() < 12 ? 0 : 1]; },
    // a.m., p.m.
    aa: function (date) { return meridiemFull[date.getHours() < 12 ? 0 : 1]; },
    Mo: ordinalFormatter('M'),
    Do: ordinalFormatter('D'),
    DDDo: ordinalFormatter('DDD'),
    do: ordinalFormatter('d'),
    Qo: ordinalFormatter('Q'),
    Wo: ordinalFormatter('W'),
};
// Generate ordinal version of formatters: M -> Mo, D -> Do, etc.
function ordinalFormatter(formatterToken) {
    return function (date) { return ordinal(defaultFormatters[formatterToken](date)); };
}
function ordinal(number) {
    var rem100 = number % 100;
    if (rem100 > 20 || rem100 < 10) {
        switch (rem100 % 10) {
            case 1:
                return number + 'st';
            case 2:
                return number + 'nd';
            case 3:
                return number + 'rd';
        }
    }
    return number + 'th';
}
function toLower(s) {
    return s.toLowerCase();
}
var months3charLower = months3char.map(toLower);
var monthsFullLower = monthsFull.map(toLower);
var parsers = {
    // Month: Jan, Feb, ..., Dec
    MMM: [
        months3charLower.join('|'),
        function (date, value) {
            var index = months3charLower.indexOf(value.toLowerCase());
            date.setMonth(index);
        },
    ],
    // Month: January, February, ..., December
    MMMM: [
        monthsFullLower.join('|'),
        function (date, value) {
            value = value.toLowerCase();
            var index = monthsFullLower.indexOf(value);
            date.setMonth(index);
        },
    ],
    Mo: ordinalParser('M'),
    Do: ordinalParser('D'),
    DDDo: ordinalParser('DDD'),
    do: ordinalParser('d'),
    Qo: ordinalParser('Q'),
    Wo: ordinalParser('W'),
};
// Generate ordinal version of parsers: M -> Mo, D -> Do, etc.
function ordinalParser(token) {
    return [
        '\\d+(?:st|nd|rd)',
        function (date, value) {
            defaultParsers[token][1](date, value.slice(0, -2));
        },
    ];
}
var format = createFormat(/*@__PURE__*/ extend(defaultFormatters, formatters));
var createCustomFormat = createCustomFormatFn(/*@__PURE__*/ extend(defaultFormatters, formatters));
var parse = createParse(/*@__PURE__*/ extend(defaultParsers, parsers));
var parseOrThrow = parseOrThrowWrapper(parse);

export { createCustomFormat, format, formatters, parse, parseOrThrow };
