import { c as createFormat, e as extend, a as createCustomFormatFn, b as createParse, d as defaultFormatters, f as defaultParsers, p as parseOrThrowWrapper } from '../../default-parsers-50f9e074.js';
export { k as addDate, j as addHours, g as addMilliseconds, i as addMinutes, l as addMonth, h as addSeconds, m as addYear, T as asValidDateOrNull, E as diffCalendarDate, D as diffCalendarMonth, C as diffCalendarYear, z as diffDate, y as diffHours, v as diffMilliseconds, x as diffMinutes, A as diffMonth, I as diffPreciseDate, H as diffPreciseHours, G as diffPreciseMinutes, J as diffPreciseMonth, F as diffPreciseSeconds, K as diffPreciseYear, w as diffSeconds, B as diffYear, L as formatDateIso, M as formatDateTimeIso, N as formatLocalIso, R as fromDate, S as fromDateOrThrow, Q as isValidDate, U as newValidDate, V as newValidDateOrThrow, O as parseIso, P as parseIsoOrThrow, q as resetDate, s as resetHours, o as resetISOWeek, t as resetMinutes, n as resetMonth, u as resetSeconds, r as resetYear } from '../../default-parsers-50f9e074.js';

var monthsShort = ['yan', 'fev', 'mar', 'apr', 'may', 'iyn', 'iyl', 'avg', 'sen', 'okt', 'noy', 'dek'];
var monthsFull = [
    'yanvar',
    'fevral',
    'mart',
    'aprel',
    'may',
    'iyun',
    'iyul',
    'avgust',
    'sentabr',
    'oktabr',
    'noyabr',
    'dekabr'
];
var weekdays2char = ['Yak', 'Dus', 'Ses', 'Cho', 'Pay', 'Jum', 'Sha'];
var weekdays3char = ['Yak', 'Dus', 'Ses', 'Cho', 'Pay', 'Jum', 'Sha'];
var weekdaysFull = ['yakshanba', 'dushanba', 'seshanba', 'chorshanba', 'payshanba', 'juma', 'shanba'];
var meridiemUppercase = ['TO', 'TK'];
var meridiemLowercase = ['to', 'tk'];
var meridiemFull = ['t.o.', 't.k.'];
var formatters = {
    // Month: yan, fev, ..., dek
    MMM: function (date) { return monthsShort[date.getMonth()]; },
    // Month: 'yanvar', 'fevral', ..., dekabr
    MMMM: function (date) { return monthsFull[date.getMonth()]; },
    // Day of week: Yak, Dus, ..., Sha
    dd: function (date) { return weekdays2char[date.getDay()]; },
    // Day of week: Yak, Dus, ..., Sha
    ddd: function (date) { return weekdays3char[date.getDay()]; },
    // Day of week: yakshanba, dushanba, ..., shanba
    dddd: function (date) { return weekdaysFull[date.getDay()]; },
    // TO, TK
    A: function (date) { return meridiemUppercase[date.getHours() < 12 ? 0 : 1]; },
    // to, tk
    a: function (date) { return meridiemLowercase[date.getHours() < 12 ? 0 : 1]; },
    // t.o., t.k.
    aa: function (date) { return meridiemFull[date.getHours() < 12 ? 0 : 1]; },
    Mo: ordinalFormatter('M'),
    Do: ordinalFormatter('D'),
    DDDo: ordinalFormatter('DDD'),
    do: ordinalFormatter('d'),
    Qo: ordinalFormatter('Q'),
    Wo: ordinalFormatter('W'),
};
// Generate ordinal version of formatters: M -> Mo, D -> Do, etc.
function ordinalFormatter(formatterToken) {
    return function (date) { return defaultFormatters[formatterToken](date); };
}
var parsers = {
    // Month: янв., фев., ..., дек.
    MMM: [
        monthsShort.join('|'),
        function (date, value) {
            var index = monthsShort.indexOf(value.toLowerCase());
            date.setMonth(index);
        },
    ],
    MMMM: [
        monthsFull.join('|'),
        function (date, value) {
            value = value.toLowerCase();
            var index = monthsFull.indexOf(value);
            date.setMonth(index);
        },
    ],
    Mo: ordinalParser('M'),
    Do: ordinalParser('D'),
    DDDo: ordinalParser('DDD'),
    do: ordinalParser('d'),
    Qo: ordinalParser('Q'),
    Wo: ordinalParser('W'),
};
// Generate ordinal version of parsers: M -> Mo, D -> Do, etc.
function ordinalParser(token) {
    return [
        '\\d+',
        function (date, value) {
            defaultParsers[token][1](date, value);
        },
    ];
}
var format = createFormat(extend(defaultFormatters, formatters));
var createCustomFormat = createCustomFormatFn(extend(defaultFormatters, formatters));
var parse = createParse(extend(defaultParsers, parsers));
var parseOrThrow = parseOrThrowWrapper(parse);

export { createCustomFormat, format, formatters, parse, parseOrThrow };
