import { c as createFormat, e as extend, a as createCustomFormatFn, b as createParse, d as defaultFormatters, f as defaultParsers, p as parseOrThrowWrapper } from '../../default-parsers-50f9e074.js';
export { k as addDate, j as addHours, g as addMilliseconds, i as addMinutes, l as addMonth, h as addSeconds, m as addYear, T as asValidDateOrNull, E as diffCalendarDate, D as diffCalendarMonth, C as diffCalendarYear, z as diffDate, y as diffHours, v as diffMilliseconds, x as diffMinutes, A as diffMonth, I as diffPreciseDate, H as diffPreciseHours, G as diffPreciseMinutes, J as diffPreciseMonth, F as diffPreciseSeconds, K as diffPreciseYear, w as diffSeconds, B as diffYear, L as formatDateIso, M as formatDateTimeIso, N as formatLocalIso, R as fromDate, S as fromDateOrThrow, Q as isValidDate, U as newValidDate, V as newValidDateOrThrow, O as parseIso, P as parseIsoOrThrow, q as resetDate, s as resetHours, o as resetISOWeek, t as resetMinutes, n as resetMonth, u as resetSeconds, r as resetYear } from '../../default-parsers-50f9e074.js';

var monthsShort = ['янв', 'фев', 'мар', 'апр', 'май', 'июн', 'июл', 'авг', 'сен', 'окт', 'ноя', 'дек'];
var monthsFull = [
    'январ',
    'феврал',
    'март',
    'апрел',
    'май',
    'июн',
    'июл',
    'август',
    'сентябр',
    'октябр',
    'ноябр',
    'декабр'
];
var weekdays2char = ['Якш', 'Душ', 'Сеш', 'Чор', 'Пай', 'Жум', 'Шан'];
var weekdays3char = ['Якш', 'Душ', 'Сеш', 'Чор', 'Пай', 'Жум', 'Шан'];
var weekdaysFull = ['якшанба', 'душанба', 'сешанба', 'чоршанба', 'пайшанба', 'жума', 'шанба'];
var meridiemUppercase = ['ТО', 'ТК'];
var meridiemLowercase = ['то', 'тк'];
var meridiemFull = ['т.о.', 'т.к.'];
var formatters = {
    // Month: янв, фев, ..., дек
    MMM: function (date) { return monthsShort[date.getMonth()]; },
    // Month: 'январ', 'феврал', ..., декабр
    MMMM: function (date) { return monthsFull[date.getMonth()]; },
    // Day of week: Якш, Душ, ..., Шан
    dd: function (date) { return weekdays2char[date.getDay()]; },
    // Day of week: Якш, Душ, ..., Шан
    ddd: function (date) { return weekdays3char[date.getDay()]; },
    // Day of week: якшанба, душанба, ..., шанба
    dddd: function (date) { return weekdaysFull[date.getDay()]; },
    // TO, TK
    A: function (date) { return meridiemUppercase[date.getHours() < 12 ? 0 : 1]; },
    // то, тк
    a: function (date) { return meridiemLowercase[date.getHours() < 12 ? 0 : 1]; },
    // т.о., т.к.
    aa: function (date) { return meridiemFull[date.getHours() < 12 ? 0 : 1]; },
    Mo: ordinalFormatter('M'),
    Do: ordinalFormatter('D'),
    DDDo: ordinalFormatter('DDD'),
    do: ordinalFormatter('d'),
    Qo: ordinalFormatter('Q'),
    Wo: ordinalFormatter('W'),
};
function ordinalFormatter(formatterToken) {
    return function (date) { return defaultFormatters[formatterToken](date); };
}
var parsers = {
    MMM: [
        monthsShort.join('|'),
        function (date, value) {
            var index = monthsShort.indexOf(value.toLowerCase());
            date.setMonth(index);
        },
    ],
    MMMM: [
        monthsFull.join('|'),
        function (date, value) {
            value = value.toLowerCase();
            var index = monthsFull.indexOf(value);
            date.setMonth(index);
        },
    ],
    Mo: ordinalParser('M'),
    Do: ordinalParser('D'),
    DDDo: ordinalParser('DDD'),
    do: ordinalParser('d'),
    Qo: ordinalParser('Q'),
    Wo: ordinalParser('W'),
};
// Generate ordinal version of parsers: M -> Mo, D -> Do, etc.
function ordinalParser(token) {
    return [
        '\\d+',
        function (date, value) {
            defaultParsers[token][1](date, value);
        },
    ];
}
var format = createFormat(extend(defaultFormatters, formatters));
var createCustomFormat = createCustomFormatFn(extend(defaultFormatters, formatters));
var parse = createParse(extend(defaultParsers, parsers));
var parseOrThrow = parseOrThrowWrapper(parse);

export { createCustomFormat, format, formatters, parse, parseOrThrow };
