'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var defaultParsers = require('../../default-parsers-24935f60.js');

// Note: in Danish, the names of days of the week and months are lowercased.
var months3char = ['jan', 'feb', 'mar', 'apr', 'maj', 'jun', 'jul', 'aug', 'sep', 'okt', 'nov', 'dec'];
var monthsFull = [
    'januar',
    'februar',
    'marts',
    'april',
    'maj',
    'juni',
    'juli',
    'august',
    'september',
    'oktober',
    'november',
    'december',
];
var weekdays2char = ['sø', 'ma', 'ti', 'on', 'to', 'fr', 'lø'];
var weekdays3char = ['søn', 'man', 'tir', 'ons', 'tor', 'fre', 'lør'];
var weekdaysFull = ['søndag', 'mandag', 'tirsdag', 'onsdag', 'torsdag', 'fredag', 'lørdag'];
var meridiemUppercase = ['AM', 'PM'];
var meridiemLowercase = ['am', 'pm'];
var meridiemFull = ['a.m.', 'p.m.'];
var formatters = {
    // Month: jan, feb, ..., dec
    MMM: function (date) { return months3char[date.getMonth()]; },
    // Month: januar, februar, ..., december
    MMMM: function (date) { return monthsFull[date.getMonth()]; },
    // Day of week: sø, ma, ..., lø
    dd: function (date) { return weekdays2char[date.getDay()]; },
    // Day of week: søn, man, ..., lør
    ddd: function (date) { return weekdays3char[date.getDay()]; },
    // Day of week: søndag, mandag, ..., lørdag
    dddd: function (date) { return weekdaysFull[date.getDay()]; },
    // AM, PM
    A: function (date) { return meridiemUppercase[date.getHours() < 12 ? 0 : 1]; },
    // am, pm
    a: function (date) { return meridiemLowercase[date.getHours() < 12 ? 0 : 1]; },
    // a.m., p.m.
    aa: function (date) { return meridiemFull[date.getHours() < 12 ? 0 : 1]; },
    Mo: ordinalFormatter('M'),
    Do: ordinalFormatter('D'),
    DDDo: ordinalFormatter('DDD'),
    do: ordinalFormatter('d'),
    Qo: ordinalFormatter('Q'),
    Wo: ordinalFormatter('W'),
};
// Generate ordinal version of formatters: M -> Mo, D -> Do, etc.
function ordinalFormatter(formatterToken) {
    return function (date) { return ordinal(defaultParsers.defaultFormatters[formatterToken](date)); };
}
function ordinal(number) {
    return number + '.';
}
function toLower(s) {
    return s.toLowerCase();
}
var months3charLower = months3char.map(toLower);
var monthsFullLower = monthsFull.map(toLower);
var parsers = {
    // Month: jan, feb, ..., dec
    MMM: [
        months3charLower.join('|'),
        function (date, value) {
            var index = months3charLower.indexOf(value.toLowerCase());
            date.setMonth(index);
        },
    ],
    // Month: januar, februar, ..., december
    MMMM: [
        monthsFullLower.join('|'),
        function (date, value) {
            value = value.toLowerCase();
            var index = monthsFullLower.indexOf(value);
            date.setMonth(index);
        },
    ],
    Mo: ordinalParser('M'),
    Do: ordinalParser('D'),
    DDDo: ordinalParser('DDD'),
    do: ordinalParser('d'),
    Qo: ordinalParser('Q'),
    Wo: ordinalParser('W'),
};
// Generate ordinal version of parsers: M -> Mo, D -> Do, etc.
function ordinalParser(token) {
    return [
        '\\d+(?:[.])',
        function (date, value) {
            defaultParsers.defaultParsers[token][1](date, value.slice(0, -1));
        },
    ];
}
var format = defaultParsers.createFormat(defaultParsers.extend(defaultParsers.defaultFormatters, formatters));
var createCustomFormat = defaultParsers.createCustomFormatFn(defaultParsers.extend(defaultParsers.defaultFormatters, formatters));
var parse = defaultParsers.createParse(defaultParsers.extend(defaultParsers.defaultParsers, parsers));
var parseOrThrow = defaultParsers.parseOrThrowWrapper(parse);

exports.addDate = defaultParsers.addDate;
exports.addHours = defaultParsers.addHours;
exports.addMilliseconds = defaultParsers.addMilliseconds;
exports.addMinutes = defaultParsers.addMinutes;
exports.addMonth = defaultParsers.addMonth;
exports.addSeconds = defaultParsers.addSeconds;
exports.addYear = defaultParsers.addYear;
exports.asValidDateOrNull = defaultParsers.asValidDateOrNull;
exports.diffCalendarDate = defaultParsers.diffCalendarDate;
exports.diffCalendarMonth = defaultParsers.diffCalendarMonth;
exports.diffCalendarYear = defaultParsers.diffCalendarYear;
exports.diffDate = defaultParsers.diffDate;
exports.diffHours = defaultParsers.diffHours;
exports.diffMilliseconds = defaultParsers.diffMilliseconds;
exports.diffMinutes = defaultParsers.diffMinutes;
exports.diffMonth = defaultParsers.diffMonth;
exports.diffPreciseDate = defaultParsers.diffPreciseDate;
exports.diffPreciseHours = defaultParsers.diffPreciseHours;
exports.diffPreciseMinutes = defaultParsers.diffPreciseMinutes;
exports.diffPreciseMonth = defaultParsers.diffPreciseMonth;
exports.diffPreciseSeconds = defaultParsers.diffPreciseSeconds;
exports.diffPreciseYear = defaultParsers.diffPreciseYear;
exports.diffSeconds = defaultParsers.diffSeconds;
exports.diffYear = defaultParsers.diffYear;
exports.formatDateIso = defaultParsers.formatDateIso;
exports.formatDateTimeIso = defaultParsers.formatDateTimeIso;
exports.formatLocalIso = defaultParsers.formatLocalIso;
exports.fromDate = defaultParsers.fromDate;
exports.fromDateOrThrow = defaultParsers.fromDateOrThrow;
exports.isValidDate = defaultParsers.isValidDate;
exports.newValidDate = defaultParsers.newValidDate;
exports.newValidDateOrThrow = defaultParsers.newValidDateOrThrow;
exports.parseIso = defaultParsers.parseIso;
exports.parseIsoOrThrow = defaultParsers.parseIsoOrThrow;
exports.resetDate = defaultParsers.resetDate;
exports.resetHours = defaultParsers.resetHours;
exports.resetISOWeek = defaultParsers.resetISOWeek;
exports.resetMinutes = defaultParsers.resetMinutes;
exports.resetMonth = defaultParsers.resetMonth;
exports.resetSeconds = defaultParsers.resetSeconds;
exports.resetYear = defaultParsers.resetYear;
exports.createCustomFormat = createCustomFormat;
exports.format = format;
exports.formatters = formatters;
exports.parse = parse;
exports.parseOrThrow = parseOrThrow;
