'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var defaultParsers = require('../../default-parsers-24935f60.js');

// http://new.gramota.ru/spravka/buro/search-answer?s=242637
var monthsShort = ['янв.', 'фев.', 'март', 'апр.', 'май', 'июнь', 'июль', 'авг.', 'сент.', 'окт.', 'нояб.', 'дек.'];
var monthsFull = [
    'январь',
    'февраль',
    'март',
    'апрель',
    'май',
    'июнь',
    'июль',
    'август',
    'сентябрь',
    'октябрь',
    'ноябрь',
    'декабрь',
];
var monthsGenitive = [
    'января',
    'февраля',
    'марта',
    'апреля',
    'мая',
    'июня',
    'июля',
    'августа',
    'сентября',
    'октября',
    'ноября',
    'декабря',
];
var weekdays2char = ['вс', 'пн', 'вт', 'ср', 'чт', 'пт', 'сб'];
var weekdays3char = ['вск', 'пнд', 'втр', 'срд', 'чтв', 'птн', 'суб'];
var weekdaysFull = ['воскресенье', 'понедельник', 'вторник', 'среда', 'четверг', 'пятница', 'суббота'];
var meridiem = ['ночи', 'утра', 'дня', 'вечера'];
function timeOfDay(date) {
    var hours = date.getHours();
    if (hours >= 17) {
        return meridiem[3];
    }
    else if (hours >= 12) {
        return meridiem[2];
    }
    else if (hours >= 4) {
        return meridiem[1];
    }
    else {
        return meridiem[0];
    }
}
var formatters = {
    // Month: янв., фев., ..., дек.
    MMM: function (date) { return monthsShort[date.getMonth()]; },
    // Month: январь, февраль, ..., декабрь
    MMMM: function (date, index, tokens) {
        var month = date.getMonth();
        if (tokens === undefined || index === undefined || index < 2)
            return monthsFull[month];
        var prevToken = tokens[index - 2];
        if ((typeof tokens[index - 1] === 'string' && prevToken === defaultParsers.defaultFormatters.D) ||
            prevToken === defaultParsers.defaultFormatters.DD) {
            // Generate formatters like 'D[some string]MMMM',
            // where month is in the genitive case: января, февраля, ..., декабря
            return monthsGenitive[month];
        }
        return monthsFull[month];
    },
    // Day of week: вс, пн, ..., сб
    dd: function (date) { return weekdays2char[date.getDay()]; },
    // Day of week: вск, пнд, ..., суб
    ddd: function (date) { return weekdays3char[date.getDay()]; },
    // Day of week: воскресенье, понедельник, ..., суббота
    dddd: function (date) { return weekdaysFull[date.getDay()]; },
    // Time of day: ночи, утра, дня, вечера
    A: timeOfDay,
    a: timeOfDay,
    aa: timeOfDay,
    // Generate ordinal version of formatters: M -> Mo, DDD -> DDDo, etc.
    Do: function (date) { return defaultParsers.defaultFormatters['D'](date) + '-е'; },
    Wo: function (date) { return defaultParsers.defaultFormatters['W'](date) + '-й'; },
    Mo: function (date) { return defaultParsers.defaultFormatters['M'](date) + '-й'; },
    DDDo: function (date) { return defaultParsers.defaultFormatters['DDD'](date) + '-й'; },
    do: function (date) { return defaultParsers.defaultFormatters['d'](date) + '-й'; },
    Qo: function (date) { return defaultParsers.defaultFormatters['Q'](date) + '-й'; },
    'Do MMMM': function (date) { return formatters['Do'](date) + ' ' + monthsGenitive[date.getMonth()]; },
};
var parsers = {
    // Month: янв., фев., ..., дек.
    MMM: [
        monthsShort.join('|'),
        function (date, value) {
            var index = monthsShort.indexOf(value.toLowerCase());
            date.setMonth(index);
        },
    ],
    // Month: январь, февраль, ..., декабрь
    // ... января, февраля, ..., декабря
    MMMM: [
        monthsFull.concat(monthsGenitive).join('|'),
        function (date, value) {
            value = value.toLowerCase();
            var index = monthsFull.indexOf(value);
            if (index < 0)
                index = monthsGenitive.indexOf(value);
            date.setMonth(index);
        },
    ],
    // Generate ordinal version of parsers by cutting the suffix
    Do: ordinalParser('D', '-е'),
    Wo: ordinalParser('W', '-й'),
    Mo: ordinalParser('M', '-й'),
    DDDo: ordinalParser('DDD', '-й'),
    do: ordinalParser('d', '-й'),
    Qo: ordinalParser('Q', '-й'),
};
function ordinalParser(token, suffix) {
    return [
        '\\d+' + suffix,
        function (date, value) {
            defaultParsers.defaultParsers[token][1](date, value.slice(0, -suffix.length));
        },
    ];
}
var format = defaultParsers.createFormat(defaultParsers.extend(defaultParsers.defaultFormatters, formatters));
var createCustomFormat = defaultParsers.createCustomFormatFn(defaultParsers.extend(defaultParsers.defaultFormatters, formatters));
var parse = defaultParsers.createParse(defaultParsers.extend(defaultParsers.defaultParsers, parsers));
var parseOrThrow = defaultParsers.parseOrThrowWrapper(parse);

exports.addDate = defaultParsers.addDate;
exports.addHours = defaultParsers.addHours;
exports.addMilliseconds = defaultParsers.addMilliseconds;
exports.addMinutes = defaultParsers.addMinutes;
exports.addMonth = defaultParsers.addMonth;
exports.addSeconds = defaultParsers.addSeconds;
exports.addYear = defaultParsers.addYear;
exports.asValidDateOrNull = defaultParsers.asValidDateOrNull;
exports.diffCalendarDate = defaultParsers.diffCalendarDate;
exports.diffCalendarMonth = defaultParsers.diffCalendarMonth;
exports.diffCalendarYear = defaultParsers.diffCalendarYear;
exports.diffDate = defaultParsers.diffDate;
exports.diffHours = defaultParsers.diffHours;
exports.diffMilliseconds = defaultParsers.diffMilliseconds;
exports.diffMinutes = defaultParsers.diffMinutes;
exports.diffMonth = defaultParsers.diffMonth;
exports.diffPreciseDate = defaultParsers.diffPreciseDate;
exports.diffPreciseHours = defaultParsers.diffPreciseHours;
exports.diffPreciseMinutes = defaultParsers.diffPreciseMinutes;
exports.diffPreciseMonth = defaultParsers.diffPreciseMonth;
exports.diffPreciseSeconds = defaultParsers.diffPreciseSeconds;
exports.diffPreciseYear = defaultParsers.diffPreciseYear;
exports.diffSeconds = defaultParsers.diffSeconds;
exports.diffYear = defaultParsers.diffYear;
exports.formatDateIso = defaultParsers.formatDateIso;
exports.formatDateTimeIso = defaultParsers.formatDateTimeIso;
exports.formatLocalIso = defaultParsers.formatLocalIso;
exports.fromDate = defaultParsers.fromDate;
exports.fromDateOrThrow = defaultParsers.fromDateOrThrow;
exports.isValidDate = defaultParsers.isValidDate;
exports.newValidDate = defaultParsers.newValidDate;
exports.newValidDateOrThrow = defaultParsers.newValidDateOrThrow;
exports.parseIso = defaultParsers.parseIso;
exports.parseIsoOrThrow = defaultParsers.parseIsoOrThrow;
exports.resetDate = defaultParsers.resetDate;
exports.resetHours = defaultParsers.resetHours;
exports.resetISOWeek = defaultParsers.resetISOWeek;
exports.resetMinutes = defaultParsers.resetMinutes;
exports.resetMonth = defaultParsers.resetMonth;
exports.resetSeconds = defaultParsers.resetSeconds;
exports.resetYear = defaultParsers.resetYear;
exports.createCustomFormat = createCustomFormat;
exports.format = format;
exports.formatters = formatters;
exports.parse = parse;
exports.parseOrThrow = parseOrThrow;
