'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var defaultParsers = require('../../default-parsers-24935f60.js');

var monthsShort = ['yan', 'fev', 'mar', 'apr', 'may', 'iyn', 'iyl', 'avg', 'sen', 'okt', 'noy', 'dek'];
var monthsFull = [
    'yanvar',
    'fevral',
    'mart',
    'aprel',
    'may',
    'iyun',
    'iyul',
    'avgust',
    'sentabr',
    'oktabr',
    'noyabr',
    'dekabr'
];
var weekdays2char = ['Yak', 'Dus', 'Ses', 'Cho', 'Pay', 'Jum', 'Sha'];
var weekdays3char = ['Yak', 'Dus', 'Ses', 'Cho', 'Pay', 'Jum', 'Sha'];
var weekdaysFull = ['yakshanba', 'dushanba', 'seshanba', 'chorshanba', 'payshanba', 'juma', 'shanba'];
var meridiemUppercase = ['TO', 'TK'];
var meridiemLowercase = ['to', 'tk'];
var meridiemFull = ['t.o.', 't.k.'];
var formatters = {
    // Month: yan, fev, ..., dek
    MMM: function (date) { return monthsShort[date.getMonth()]; },
    // Month: 'yanvar', 'fevral', ..., dekabr
    MMMM: function (date) { return monthsFull[date.getMonth()]; },
    // Day of week: Yak, Dus, ..., Sha
    dd: function (date) { return weekdays2char[date.getDay()]; },
    // Day of week: Yak, Dus, ..., Sha
    ddd: function (date) { return weekdays3char[date.getDay()]; },
    // Day of week: yakshanba, dushanba, ..., shanba
    dddd: function (date) { return weekdaysFull[date.getDay()]; },
    // TO, TK
    A: function (date) { return meridiemUppercase[date.getHours() < 12 ? 0 : 1]; },
    // to, tk
    a: function (date) { return meridiemLowercase[date.getHours() < 12 ? 0 : 1]; },
    // t.o., t.k.
    aa: function (date) { return meridiemFull[date.getHours() < 12 ? 0 : 1]; },
    Mo: ordinalFormatter('M'),
    Do: ordinalFormatter('D'),
    DDDo: ordinalFormatter('DDD'),
    do: ordinalFormatter('d'),
    Qo: ordinalFormatter('Q'),
    Wo: ordinalFormatter('W'),
};
// Generate ordinal version of formatters: M -> Mo, D -> Do, etc.
function ordinalFormatter(formatterToken) {
    return function (date) { return defaultParsers.defaultFormatters[formatterToken](date); };
}
var parsers = {
    // Month: янв., фев., ..., дек.
    MMM: [
        monthsShort.join('|'),
        function (date, value) {
            var index = monthsShort.indexOf(value.toLowerCase());
            date.setMonth(index);
        },
    ],
    MMMM: [
        monthsFull.join('|'),
        function (date, value) {
            value = value.toLowerCase();
            var index = monthsFull.indexOf(value);
            date.setMonth(index);
        },
    ],
    Mo: ordinalParser('M'),
    Do: ordinalParser('D'),
    DDDo: ordinalParser('DDD'),
    do: ordinalParser('d'),
    Qo: ordinalParser('Q'),
    Wo: ordinalParser('W'),
};
// Generate ordinal version of parsers: M -> Mo, D -> Do, etc.
function ordinalParser(token) {
    return [
        '\\d+',
        function (date, value) {
            defaultParsers.defaultParsers[token][1](date, value);
        },
    ];
}
var format = defaultParsers.createFormat(defaultParsers.extend(defaultParsers.defaultFormatters, formatters));
var createCustomFormat = defaultParsers.createCustomFormatFn(defaultParsers.extend(defaultParsers.defaultFormatters, formatters));
var parse = defaultParsers.createParse(defaultParsers.extend(defaultParsers.defaultParsers, parsers));
var parseOrThrow = defaultParsers.parseOrThrowWrapper(parse);

exports.addDate = defaultParsers.addDate;
exports.addHours = defaultParsers.addHours;
exports.addMilliseconds = defaultParsers.addMilliseconds;
exports.addMinutes = defaultParsers.addMinutes;
exports.addMonth = defaultParsers.addMonth;
exports.addSeconds = defaultParsers.addSeconds;
exports.addYear = defaultParsers.addYear;
exports.asValidDateOrNull = defaultParsers.asValidDateOrNull;
exports.diffCalendarDate = defaultParsers.diffCalendarDate;
exports.diffCalendarMonth = defaultParsers.diffCalendarMonth;
exports.diffCalendarYear = defaultParsers.diffCalendarYear;
exports.diffDate = defaultParsers.diffDate;
exports.diffHours = defaultParsers.diffHours;
exports.diffMilliseconds = defaultParsers.diffMilliseconds;
exports.diffMinutes = defaultParsers.diffMinutes;
exports.diffMonth = defaultParsers.diffMonth;
exports.diffPreciseDate = defaultParsers.diffPreciseDate;
exports.diffPreciseHours = defaultParsers.diffPreciseHours;
exports.diffPreciseMinutes = defaultParsers.diffPreciseMinutes;
exports.diffPreciseMonth = defaultParsers.diffPreciseMonth;
exports.diffPreciseSeconds = defaultParsers.diffPreciseSeconds;
exports.diffPreciseYear = defaultParsers.diffPreciseYear;
exports.diffSeconds = defaultParsers.diffSeconds;
exports.diffYear = defaultParsers.diffYear;
exports.formatDateIso = defaultParsers.formatDateIso;
exports.formatDateTimeIso = defaultParsers.formatDateTimeIso;
exports.formatLocalIso = defaultParsers.formatLocalIso;
exports.fromDate = defaultParsers.fromDate;
exports.fromDateOrThrow = defaultParsers.fromDateOrThrow;
exports.isValidDate = defaultParsers.isValidDate;
exports.newValidDate = defaultParsers.newValidDate;
exports.newValidDateOrThrow = defaultParsers.newValidDateOrThrow;
exports.parseIso = defaultParsers.parseIso;
exports.parseIsoOrThrow = defaultParsers.parseIsoOrThrow;
exports.resetDate = defaultParsers.resetDate;
exports.resetHours = defaultParsers.resetHours;
exports.resetISOWeek = defaultParsers.resetISOWeek;
exports.resetMinutes = defaultParsers.resetMinutes;
exports.resetMonth = defaultParsers.resetMonth;
exports.resetSeconds = defaultParsers.resetSeconds;
exports.resetYear = defaultParsers.resetYear;
exports.createCustomFormat = createCustomFormat;
exports.format = format;
exports.formatters = formatters;
exports.parse = parse;
exports.parseOrThrow = parseOrThrow;
