import { DiffUnitFn } from '../utils/basic-types';
/**
 * Returns the number of years between two dates.
 * Months, days and time are ignored
 * @example
 * diffCalendarYear(parseIso('2020-01-01'), parseIso('2019-12-31')) // 1 = 2020 - 2019
 */
export declare function diffCalendarYear<T extends Date | null>(d1: T, d2: T): DiffUnitFn<T>;
/**
 * Returns the number of months between two dates.
 * Days and time are ignored
 * @example
 * diffCalendarMonth(parseIso('2020-05-01'), parseIso('2020-03-31')) // 2 = 5 - 3
 */
export declare function diffCalendarMonth<T extends Date | null>(d1: T, d2: T): DiffUnitFn<T>;
/**
 * Returns the number of days between two dates.
 * Time is ignored
 * @example
 * diffCalendarDate(parseIso('2020-05-01'), parseIso('2020-04-20')) // 11 = 10 days in april and 1 in march
 */
export declare function diffCalendarDate<T extends Date | null>(d1: T, d2: T): DiffUnitFn<T>;
