import { DiffUnitFn } from '../utils/basic-types';
/**
 * The number of milliseconds between two dates
 * @example
 * diffMilliseconds(parseIso('2020-01-02'), parseIso('2020-01-01')) // 86400000 = milliseconds in one day
 */
export declare function diffMilliseconds<T extends Date | null>(d1: T, d2: T): DiffUnitFn<T>;
/**
 * The number of seconds between two dates
 * @example
 * diffSeconds(parseIso('2020-01-02'), parseIso('2020-01-01')) // 86400 = seconds in one day
 */
export declare function diffSeconds<T extends Date | null>(d1: T, d2: T): DiffUnitFn<T>;
/**
 * The number of minutes between two dates
 * @example
 * diffMinutes(parseIso('2020-01-02'), parseIso('2020-01-01')) // 1440 = minutes in one day
 * diffMinutes(parseIso('2020-01-01T12:30:01'), parseIso('2020-01-01T12:31:00')) // 0 = only 59 seconds left
 */
export declare function diffMinutes<T extends Date | null>(d1: T, d2: T): DiffUnitFn<T>;
export declare function diffHours<T extends Date | null>(d1: T, d2: T): DiffUnitFn<T>;
export declare function diffDate<T extends Date | null>(d1: T, d2: T): DiffUnitFn<T>;
export declare function diffMonth<T extends Date | null>(d1: T, d2: T): DiffUnitFn<T>;
export declare function diffYear<T extends Date | null>(d1: T, d2: T): DiffUnitFn<T>;
